/*----------------------------------------------------------------------------*/
/*                                                                            */
/* Copyright (c) 1995, 2004 IBM Corporation. All rights reserved.             */
/* Copyright (c) 2005-2018 Rexx Language Association. All rights reserved.    */
/*                                                                            */
/* This program and the accompanying materials are made available under       */
/* the terms of the Common Public License v1.0 which accompanies this         */
/* distribution. A copy is also available at the following address:           */
/* https://www.oorexx.org/license.html                                        */
/*                                                                            */
/* Redistribution and use in source and binary forms, with or                 */
/* without modification, are permitted provided that the following            */
/* conditions are met:                                                        */
/*                                                                            */
/* Redistributions of source code must retain the above copyright             */
/* notice, this list of conditions and the following disclaimer.              */
/* Redistributions in binary form must reproduce the above copyright          */
/* notice, this list of conditions and the following disclaimer in            */
/* the documentation and/or other materials provided with the distribution.   */
/*                                                                            */
/* Neither the name of Rexx Language Association nor the names                */
/* of its contributors may be used to endorse or promote products             */
/* derived from this software without specific prior written permission.      */
/*                                                                            */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS        */
/* "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          */
/* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS          */
/* FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   */
/* OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,      */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED   */
/* TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,        */
/* OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY     */
/* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING    */
/* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS         */
/* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.               */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/******************************************************************************/
/* REXX Kernel                                                                */
/*                                                                            */
/* Windows memory management                                                  */
/******************************************************************************/

#include "RexxCore.h"
#include "RexxMemory.hpp"
#include "ActivityManager.hpp"
#include "SystemInterpreter.hpp"
#include "Memory.hpp"


/**
 * Allocate a block of memory suitable to pass back to
 * another program as a return result
 *
 * @param size   The memory size to allocate.
 *
 * @return The pointer to the allocated memory.
 */
void *SystemInterpreter::allocateResultMemory(size_t size)
{
    return (void *)GlobalAlloc(GMEM_FIXED, size);
}


/**
 * Release a block of memory given to us by an outside
 * agent as a return result
 *
 * @param memoryBlock
 *               The block of memory to release.
 */
void SystemInterpreter::releaseResultMemory(void *memoryBlock)
{
    // The DB2 interfaces were using the wrong allocation method for the
    // storage.  So, of course, we're the ones who have to adjust.
    if (GlobalSize(memoryBlock))
    {
        GlobalFree(memoryBlock);
    }
    else
    {
        free(memoryBlock);
    }
}


/**
 * Allocate a segment of memory using the system-defined mechanism
 *
 * @param size   The size of the segment
 *
 * @return The pointer to the segment or NULL is there is an allocation error.
 */
void *SystemInterpreter::allocateSegmentMemory(size_t size)
{
    return (void *)GlobalAlloc(GMEM_FIXED, size);

}


/**
 * Return a heap segment back to the system.
 *
 * @param segmentBlock
 *               The pointer to the block to release.
 */
void SystemInterpreter::releaseSegmentMemory(void *segmentBlock)
{
    GlobalFree(segmentBlock);
}
